/**************************************************************
 * Broker.java
 *
 *
 *
 **************************************************************/

package hotTip;

// GUI + misc
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.JPasswordField;
import java.io.*;
import java.util.*;
import java.math.BigInteger;

//  Crypto
import java.security.*;
import java.security.cert.*;
import java.security.spec.*;
import java.security.interfaces.*;
import javax.security.auth.x500.*;

import javax.crypto.*;
import javax.crypto.spec.*;
import javax.crypto.interfaces.*;

// socket
import java.net.*;
import javax.net.ssl.*;

// IAIK security toolkit
import iaik.x509.*;
import iaik.asn1.structures.AlgorithmID;


public class Broker {

    /**********************[GUI-related vars]*******************/
    BrokerPanel _panel;
    BrokerActivity _activityPanel;
    CardLayout _layout;
    JFrame _appFrame;    

    BrokerThread _thread;
    private JTextArea _outputArea;
    
    /********************[security vars]********************/
    // Some hints: security related fields.
    private String BROKER_KEYSTORE = null;
    private char[] BROKER_KEYSTORE_PASSWORD = null;
    private char[] BROKER_KEY_PASSWORD = null;
    
    /*************************[communication vars]*******************/
    //private SSLServerSocket brokerSocket = null;
    private ServerSocket brokerSocket = null;
    private Socket cliSock = null;
    
    private SecureRandom secureRandom;
    private KeyStore brokerKeyStore;
    private KeyManagerFactory keyManagerFactory;
    private TrustManagerFactory trustManagerFactory;
    private SSLContext sslContext;
    private SSLServerSocketFactory serverSocketFactory;
    
    /**********************************************************************/
    public int port;
    public String hostName;

    private String brokerKeyStoreName = null;
	
    private char[] brokerKeyStorePassword = null;

    private static boolean brkrSktSetUp = false;
    
    /**********************************************************************
     *
     *
     *
     *
     *
     **********************************************************************/
    public Broker( ) {
	
	InetAddress serverAddr = null;
	
	try {	    
	    serverAddr = InetAddress.getByName(null); }

	catch(UnknownHostException uhe) {
	    System.err.println( uhe.getMessage() );
	    uhe.printStackTrace(); }
                
	hostName = serverAddr.getHostName();	
	
	_panel = null;
	_activityPanel = null;
	_layout = null;
	_appFrame = null;
	
	try {
	    initialize(); } 
	
	catch (Exception e) {
	    System.err.println("Broker error: " + e.getMessage());
	    e.printStackTrace(); }
	
	_layout.show(_appFrame.getContentPane(), "BrokerPanel");
    }   

    /**********************************************************************
     *
     *
     *
     *
     *
     **********************************************************************/    
    private void initialize() throws Exception {
	
	_appFrame = new JFrame("Broker");
	_layout = new CardLayout();
	
	_appFrame.getContentPane().setLayout(_layout);
	_panel = new BrokerPanel(this);
	_appFrame.getContentPane().add(_panel, "BrokerPanel");
	
	_activityPanel = new BrokerActivity(this);
	_appFrame.getContentPane().add(_activityPanel, "BrokerActivityPanel");	

	_appFrame.addWindowListener(new WindowAdapter() {
		public void windowClosing(WindowEvent e) {
		    quit();
		}
	    });
    }

    public void run() {
        _appFrame.pack();
        _appFrame.setVisible(true);
    }       

    /****************************************************************
     *
     *
     *
     *
     ***************************************************************/
    void quit() {

	System.exit(0);	
    }

    /****************************************************************
     *
     *
     *
     *
     *
     ***************************************************************/
    public void startup( int _port, String brkrSignedKSname, 
			 char[] brkrSignedKSpwd ) {
	
	port = _port;       
	
	brokerKeyStoreName = brkrSignedKSname;
	
	brokerKeyStorePassword = brkrSignedKSpwd;
	
        /**************************************************************/
	// set up SSLContext, KeyManager, TrustManager, 
	//        SSLServerSocketFactory,...      
	/**************************************************************/ 
	
	_layout.show(_appFrame.getContentPane(), "BrokerActivityPanel");

	try {	    
	    brokerSocket = new ServerSocket( port );
	    brkrSktSetUp = true; }

	catch(Exception e) {
	    System.err.println("Broker error: " + e.getMessage() );
	    e.printStackTrace(); }
    	
	_activityPanel.getOutputArea().append("Broker waiting on " + 
					      hostName + " port " + port );
    }


    /****************************************************************
     *
     *
     *
     *
     *
     ***************************************************************/
    public void more( ) {
	
	while( true ) {
	    
	    try {
		cliSock = brokerSocket.accept();
		_thread = new BrokerThread( this, cliSock );
		_thread.start(); }
	    
	    catch (IOException e) {
		System.err.println("Accept failed.");
		System.exit(1); }
	}
    }
    

    /**************************************************************
     *
     *
     *
     *
     **************************************************************/
    public static void main(String[] args) {
	
	Security.addProvider(new com.sun.crypto.provider.SunJCE());
	
	Broker broker = new Broker();
	broker.run();

	while( !brkrSktSetUp )
	    ;
	
	broker.more();
    }
    
    /**************************************************************
     *
     *
     *
     *
     **************************************************************/
    public int getPortNumber() {
	
	return port;
    }

    /**************************************************************
     *
     *
     *
     *
     **************************************************************/    
    public JTextArea getOutputArea() {
    
	return _activityPanel.getOutputArea();
    }
    
}
